///////////////////////////////////////////////////////////////////////////////
//! \file fcBaseXENOMAI.h:
//! \brief Header File for the fcBase API (only XENOMAI functions)
//!
//! Use this header file for developing user defined FlexCard applications.
//! See also printed documentation for more information. The data types and
//! enumerations can be found in the header file "fcBaseTypes.h".
//! 
//! Generally:
//! 
//! - Each function of this library (except some error handling functions)
//!   returns an error code. If the return value is equal to 0, no error
//!   occured. A number greater than 0 indicates an error. To get more
//!   information about it, use the error handling functions. 
//!
//! \par Overview:
//! Author:         TB
//! $Date: 2015-07-21 13:45:20 +0200 (Di, 21 Jul 2015) $ <br>
//! \par Copyright:
//! Copyright 2007-2008 STAR COOPERATION GmbH. All rights reserved.
//! \version $Revision: 18026 $
///////////////////////////////////////////////////////////////////////////////

#ifndef _FCBASE_XENOMAI_H_
#define _FCBASE_XENOMAI_H_

#ifndef _FCBASE_TYPES_H_
    #error Please include first the header file "fcBaseTypes.h"
#endif 

#ifdef WIN32
	#error This header file must not be used for Windows systems. 
#endif

// Linux
#ifndef FCBASE_API
	#define FCBASE_API
#endif
	//! Please do not use the FCBASE_EXPORTS define in your application, unless
	//! you have the source code ;-)
	#ifdef FCBASE_EXPORTS
		#define EXPORTDLL extern "C" __attribute__ ((visibility ("default")))
	#else
		#if defined(__cplusplus)
			#define EXPORTDLL extern "C"
		#else
			#define EXPORTDLL 
		#endif
	#endif

//----------------------------------------------------------------------
//          Events
//----------------------------------------------------------------------

#ifdef XENOMAI
//! THIS FUNCTION IS OBSOLETE  (Please use #fcbWaitForEventV2 now.)
//! This function makes a safe real time I/O-Control that blocks the user
//! process in kernel-space, until an event of the given type occurs or the
//! event does not appear within the specified amount of time. The driver's
//! kernel interrupt service routine then unblocks and the program routine
//! continues. Works without #fcbSetEventHandle.
//! \param hFlexCard IN] <br> Handle to a FlexCard
//! \param hEvent [IN] <br> The notification type for which event has to be wait for.
//! \param nTimeout	[IN] <br> The maximum amount of time to wait for the event.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcNotificationType
EXPORTDLL fcError FCBASE_API fcbWaitForEvent(fcHandle hFlexCard,
											 fcNotifyType hEvent,
											 fcDword nTimeout);

//! This function makes a safe real time I/O-Control that blocks the user
//! process in kernel-space, until an event of the given type occurs or the
//! event does not appear within the specified amount of time. The driver's
//! kernel interrupt service routine then unblocks and the program routine
//! continues. Works without #fcbSetEventHandle.
//! \param hFlexCard IN] <br> Handle to a FlexCard.
//! \param CC [IN] <br> Index of the communication controller.
//! \param hEvent [IN] <br> The notification type for which event has to be wait for.
//! \param nTimeout	[IN] <br> The maximum amount of time to wait for the event.
//! \return If the function succeeds, the return value is 0. If the value <> 0,
//!         please use the functions described in the section Error Handling to
//!         get extended error information.
//! \see #fcCC #fcNotificationType
EXPORTDLL fcError FCBASE_API fcbWaitForEventV2(fcHandle hFlexCard,
											   fcCC CC,
											   fcNotificationType type,
											   fcDword nTimeout);
#endif

#endif /* #define _FCBASE_XENOMAI_H_ */
