///////////////////////////////////////////////////////////////////////////////
//! \file fcBaseTypesPMC.h:
//! \brief Header file for the data types and enumerations of the fcBase PMC API
//!
//! Use this header file for developing user defined FlexCard applications. This
//! header file contains all structures, data types and enumerations which are
//! used in the fcBase PMC API. For a description of the API function, please
//! refer to the header file "fcBasePMC.h".
//!
//! See also printed documentation for more information. 
//!  
//! \par Overview:
//! Author:         WO/SH/Ba/TB/Tr/LJ
//! $Date$ <br>
//! \par Copyright:
//! Copyright 2009-2015 STAR COOPERATION GmbH. All rights reserved.
//! \version $Revision$
///////////////////////////////////////////////////////////////////////////////

#ifndef _FCBASE_TYPES_PMC_H_
#define _FCBASE_TYPES_PMC_H_

//-----------------------------------------------------------------------------
//           Initialization
//-----------------------------------------------------------------------------

//! This enumeration defines the available FlexCard bus channels. The values
//! fcBusChannel5 to fcBusChannel8 are only valid for the FlexCard PMC-II.
typedef enum fcBusChannel
{
	fcBusChannel1 = 1,	//!< The bus channel 1 is selected.
	fcBusChannel2 = 2,	//!< The bus channel 2 is selected.
	fcBusChannel3 = 3,	//!< The bus channel 3 is selected.
	fcBusChannel4 = 4,	//!< The bus channel 4 is selected.
	fcBusChannel5 = 5,	//!< The bus channel 5 is selected.
	fcBusChannel6 = 6,	//!< The bus channel 6 is selected.
	fcBusChannel7 = 7,	//!< The bus channel 7 is selected.
	fcBusChannel8 = 8,	//!< The bus channel 8 is selected.
} fcBusChannel;


//! This enumeration defines the FlexCard FlexTiny types. The FlexTiny modules
//! are small PCBs that contain physical layer components.
typedef enum fcTinyType
{
    fcTinyTypeUnknown = 0,			///< The bus type is unknown
    fcTinyTypeFlexRay,				///< The bus type is FlexRay
    fcTinyTypeCAN,					///< The bus type is CAN
    fcTinyTypeEthernet,					///< The bus type is Ethernet
    fcTinyTypeRS232,					///< The bus type is a serial RS232 connection
    fcTinyTypeCAN_LS,					///< The bus type is CAN_LS
    fcTinyTypeCAN_HS,					///< The bus type is CAN_HS
    fcTinyTypeLIN,					///< The bus type is LIN	
    fcTinyTypeK_Line,					///< The bus type is K-LINE	
    fcTinyTypeFlexRayIso,			///< The bus type is FlexRay Iso	
    fcTinyTypeCAN_HS_Iso,				///< The bus type is CAN_HS Iso	
    fcTinyTypeCAN_LS_Iso,				///< The bus type is CAN_LS Iso	
    fcTinyTypeLIN_Iso,				///< The bus type is LIN Iso	
    fcTinyTypeRS232_Iso,				///< The bus type is RS232 Iso	
    fcTinyTypeCAN_FD,				    ///< The bus type is CAN-FD
    fcTinyTypeCAN_FD_Iso,			    ///< The bus type is CAN-FD Iso
    fcTinyTypeUSB,				    ///< The bus type is USB
    fcTinyTypeBroadR_Reach,		    ///< The bus type is BroadR-Reach (Automotive Ethernet)
    fcTinyType100BASE_T1 = fcTinyTypeBroadR_Reach,		    ///< The bus type is 100BASE-T1 (Automotive Ethernet)
}fcTinyType;

//! This structure contains information about a FlexTiny.
typedef struct fcTinyInfo
{
    fcTinyType TinyType;            //!< The type of the FlexTiny.
    fcDword Reserved;               //!< Reserved for future use.
} fcTinyInfo;

//! This structure contains information about all the FlexTiny modules that
//! are mounted on a FlexCard.
typedef struct fcTinyInfoCollection
{
    fcTinyInfo info[255];           //!< Information about the mounted FlexTiny modules.
} fcTinyInfoCollection;


//-----------------------------------------------------------------------------
//          Configuration
//-----------------------------------------------------------------------------

//! This enumeration defines the conditions available for a trigger configuration.
typedef enum fcTriggerConditionPMC
{	
	fcTriggerPMCNone					= 0x00000000, //!< None.
	fcTriggerPMCIn      				= 0x00000100, //!< Input trigger. When a trigger impulse is detected a fcTriggerExInfoPacket packet is generated
	fcTriggerPMCOutOnPulse				= 0x00001000, //!< Output trigger. Impulse is generated as soon as the condition is set to the driver
	fcTriggerPMCOutOnTimeStampChanged	= 0x00002000, //!< Output trigger. Impulse is generated if the internal FlexCard time stamp was changed
	fcTriggerPMCOutOnErrorDetected		= 0x00010000, //!< Output trigger. Impulse is generated if a FlexRay error was detected
	fcTriggerPMCOutOnStartupCompleted	= 0x00020000, //!< Output trigger. Impulse is generated when the FlexRay startup was completed
	fcTriggerPMCOutOnCycleStart			= 0x00100000, //!< Output trigger. Impulse is generated at a FlexRay cycle start
} fcTriggerConditionPMC;

#endif /* #define _FCBASE_TYPES_PMC_H_ */
