#include "fc.h"

//////////////////////////////////////////////////////////////////////////
// Initialization
t_FC fc[MAXIMUMFLEXCARD] =
{
	{fcNoDevice,0,NULL,{false,false,false,false,false,false,false,false}},
	{fcNoDevice,0,NULL,{false,false,false,false,false,false,false,false}},
	{fcNoDevice,0,NULL,{false,false,false,false,false,false,false,false}},
	{fcNoDevice,0,NULL,{false,false,false,false,false,false,false,false}},
	{fcNoDevice,0,NULL,{false,false,false,false,false,false,false,false}},
	{fcNoDevice,0,NULL,{false,false,false,false,false,false,false,false}},
	{fcNoDevice,0,NULL,{false,false,false,false,false,false,false,false}},
	{fcNoDevice,0,NULL,{false,false,false,false,false,false,false,false}}
};

//////////////////////////////////////////////////////////////////////////
// Helper functions
void PrintError(fcError e, const char* szFunction)
{
	if (0 == e) return; // No error: nothing to display ...

	fcErrorCode errorCode = fcGetErrorCode(e);
	char* szErrorText = NULL;
	// Get the error text
	if (0 != fcGetErrorText(e, &szErrorText))
	{
		fprintf(g_pFile, "\n%s Couldn't retrieve error text from error "
			"(Error: 0x%08X)\n", szFunction, errorCode);
	}
	else
	{
		fprintf(g_pFile, "\nError %s: %s (ErrorCode: 0x%08X)\n",
			szFunction, szErrorText, errorCode);

		// Free the memory allocated by fcGetErrorText()
		fcFreeMemory(fcMemoryTypeString, reinterpret_cast<void*>(szErrorText));
	}
}

void PrintVersion(const fcVersionNumber& v)
{
	fprintf(g_pFile, "%d.%d.%d.%d\n", v.Major, v.Minor, v.Update, v.Release);	
}

void PrintHeader(const char* szText)
{
	fprintf(g_pFile, "\n\n");
	fprintf(g_pFile, "%s...\n", szText);
	fprintf(g_pFile, "------------------------------------\n");
}

void PrintHeaderFunctionAndCC(const char* szText, fcCC CCIndex)
{
	fprintf(g_pFile, "\n\n");
	fprintf(g_pFile, "%s  fcCC%d...\n", szText, CCIndex+1);
	fprintf(g_pFile, "------------------------------------\n");
}

std::string ReadFile(const char* szPath)
{
	std::string s;
	std::ifstream file(szPath);

	if (! file.is_open()) return s;

	char ch;
	while (file.get(ch)) s += ch;
	file.close();
    
	return s;
}

//////////////////////////////////////////////////////////////////////////
// Common API Wrapper Functions
void EnumerateFlexCards()
{
	PrintHeader("fcbGetEnumFlexCardsV3");

	unsigned int nCurrent = 0;
	fcInfoHwSw* pInfoHwSw = NULL;
	fcError e = fcbGetEnumFlexCardsV3(&pInfoHwSw, false);
	if (0 == e)
	{
		// Print out the flexcard version info
		fcInfoHwSw* pLoop = pInfoHwSw;
		while (NULL != pLoop)
		{
			// if FlexCard Id is equal to zero we got NO FlexCard in the system
			bool bFlexCardAvailable = (0 != pLoop->FlexCardId);

			if (! bFlexCardAvailable)
			{
				fprintf(g_pFile,"\n");
				fprintf(g_pFile,"FlexCard Id: not available\n");
				fprintf(g_pFile,"Device Id: not available\n");
				fprintf(g_pFile,"Serial Number: not available\n");

				// Dll Version is always available
				fprintf(g_pFile,"DLL: ");
				PrintVersion(pLoop->InfoSoftware.VersionBaseDll);
				fprintf(g_pFile,"SYS: not available\n");
				fprintf(g_pFile,"Firmware version: not available\n");
				fprintf(g_pFile,"Hardware: not available\n");
			}
			else
			{
				fprintf(g_pFile,"\nFlexCard Id: ");
				//Remember the FlexCard id to open a connection
				if (nCurrent < MAXIMUMFLEXCARD)
				{
					fc[nCurrent].Id = pLoop->FlexCardId;
					fc[nCurrent].DeviceId = pLoop->InfoHardware.DeviceId;
					nCurrent++;
				}
				fprintf(g_pFile,"%d\n", pLoop->FlexCardId);

				fprintf(g_pFile,"Busy: ");
				fprintf(g_pFile,"%s\n", (pLoop->Busy ? "true" : "false"));

				fprintf(g_pFile,"Device Id: ");
				fprintf(g_pFile,"%s\n",
					/*gDeviceIdMap[pLoop->InfoHardware.DeviceId].c_str()*/
                    deviceName[pLoop->InfoHardware.DeviceId].c_str());

				fprintf(g_pFile,"Serial Number: ");
#ifdef __linux__
				fprintf(g_pFile,"%llu\n", pLoop->InfoHardware.Serial);
#else
				fprintf(g_pFile,"%I64u\n", pLoop->InfoHardware.Serial);
#endif

				// Dll Version is always available
				fprintf(g_pFile,"DLL: ");
				PrintVersion(pLoop->InfoSoftware.VersionBaseDll);

				fprintf(g_pFile,"SYS: ");
				PrintVersion(pLoop->InfoSoftware.VersionDeviceDriver);

                std::string fullCommunicationAccess = "false";
                if (pLoop->InfoHardware.FullCommunicationControllerAccess)
                {
                    fullCommunicationAccess = "true";
                }
                
                fprintf(g_pFile, "FullCommunicationAccess: %s\n", fullCommunicationAccess.c_str());

				fprintf(g_pFile,"Firmware version: ");
				PrintVersion(pLoop->InfoHardware.VersionFirmware);

				fprintf(g_pFile,"Hardware: ");
				PrintVersion(pLoop->InfoHardware.VersionHardware);

				fprintf(g_pFile,"Useable FlexRay CCs: ");
				fprintf(g_pFile,"%d\n",
					pLoop->InfoHardware.UseableCCs.FlexRay);

				fprintf(g_pFile,"Supported FlexRay CCs: ");
				fprintf(g_pFile,"%d\n",
					pLoop->InfoHardware.SupportedCCs.FlexRay);

                fprintf(g_pFile, "Useable CAN CCs: ");
                fprintf(g_pFile, "%d\n", pLoop->InfoHardware.UseableCCs.CAN);

                fprintf(g_pFile, "Supported CAN CCs: ");
                fprintf(g_pFile, "%d\n", pLoop->InfoHardware.SupportedCCs.CAN);

				// print CC information
				fcVersionCC* pLoopCC = pLoop->InfoHardware.pVersionCC;
				while (NULL != pLoopCC)
				{
					unsigned int x = pLoopCC->CCIndex;
					x += 1;

                    fprintf(g_pFile,"CC%d Type: ",x);

                    const auto busTypeString = busTypeMapping[pLoopCC->BusType];
                    const auto ccTypeString = ccTypeName[pLoopCC->CCType];

                    fprintf(g_pFile, "%s - %s\n", busTypeString.c_str(), ccTypeString.c_str());

                    fprintf(g_pFile,"    Protocol: ");
					PrintVersion(pLoopCC->Protocol);

					pLoopCC = pLoopCC->pNext; // get the next CC 
				}
			}
			pLoop = pLoop->pNext; // get the next flexcard
		}

		fcFreeMemory(fcMemoryTypeInfoHwSw, pInfoHwSw);
	}

    fprintf(g_pFile,"\n");
}

void OpenFlexCard(unsigned int n)
{
	PrintHeader("fcbOpen");
	
	fcError e = fcbOpen(&fc[n].Handle, fc[n].Id);
	if (0 == e)
	{
		fprintf(g_pFile, "OK. A connection to the FlexCard was "
			"successfully opened.\n");
	}
    else
    { 
        fc[n].Handle = NULL;
        PrintError(e, "fcbOpen");
    }
}

void CloseFlexCard(unsigned int n)
{
	PrintHeader("fcbClose");
	
	fcError e = fcbClose(fc[n].Handle);
	fc[n].Handle = NULL;

	if (0 == e)
	{
		fprintf(g_pFile, "OK. The connection to the FlexCard was "
			"successfully closed.\n");
	}
	else
	{
		PrintError(e, "fcbClose");
	}
}

bool Receive(unsigned int n, fcPacket** pPacket)
{
	PrintHeader("fcbReceive");

	fcError e = fcbReceive(fc[n].Handle, pPacket);
    if (0 == e)
	{
		//fprintf(g_pFile,"OK.\n");
		return true;
	}
    else
	{
		PrintError(e, "fcbReceive");
	}
	return false;
}

bool FreeMemory(fcMemoryType type, void* p)
{
	//PrintHeader("fcFreeMemory");

	fcError e = fcFreeMemory(type, p);
    if (0 == e) 
	{
		//fprintf(g_pFile, "OK.\n");
		return true;
	}
    else
	{
		PrintError(e, "fcFreeMemory");
	}
	return false;
}

bool GetCurrentHighResTimeStamp(unsigned int n, unsigned long long* pTimestamp)
{
	//PrintHeader("fcbGetCurrentHighResTimeStamp");

	if (NULL == pTimestamp)
	{
		return false;
	}

	fcQuad quTimeStamp = 0LL;
	fcQuad quPerfCounter = 0LL;

	fcError e = fcbGetCurrentHighResTimeStamp(fc[n].Handle, &quTimeStamp,
		&quPerfCounter);
	if (0 == e)
	{
		*pTimestamp = quTimeStamp;
		//fprintf(g_pFile, "OK.\n");
		return true;
	}
	else 
	{
		PrintError(e, "fcbGetCurrentHighResTimeStamp");
	}
	return false;
}

//////////////////////////////////////////////////////////////////////////
// Termination API Wrapper Functions
bool SetBusTerminationCC(unsigned int n, fcCC eCC, bool bTermination)
{
	//PrintHeaderFunctionAndCC("fcbSetBusTerminationCc", eCC);

	fcError e = fcbSetBusTerminationCc(fc[n].Handle, fcBusTypeCAN, eCC,
		fcChannelNone, bTermination);
	if (0 == e)
	{
		//fprintf(g_pFile, "Ok.\n");
		return true;
	}
	else 
	{
		PrintError(e, "fcbSetBusTerminationCc");
	}
	return false;
}

//////////////////////////////////////////////////////////////////////////
// CAN API Wrapper Functions
bool CANSetCcConfiguration(unsigned int n, fcCC eCC, fcCANCcConfig pCAN)
{
	//PrintHeaderFunctionAndCC("fcbCANSetCcConfiguration", eCC);

	fcError e = fcbCANSetCcConfiguration(fc[n].Handle, eCC, pCAN);
	if (0 == e)
	{
		//fprintf(g_pFile, "Ok.\n");
		return true;
	}
	else 
	{
		PrintError(e, "fcbCANSetCcConfiguration");
	}
	return false;
}

bool CANFDSetCcConfiguration(unsigned int n, fcCC eCC, fcCANFDCcConfig pCAN)
{
	//PrintHeaderFunctionAndCC("fcbCANFDSetCcConfiguration", eCC);

	fcError e = fcbCANFDSetCcConfiguration(fc[n].Handle, eCC, pCAN);
	if (0 == e)
	{
		//fprintf(g_pFile, "Ok.\n");
		return true;
	}
	else 
	{
		PrintError(e, "fcbCANFDSetCcConfiguration");
	}
	return false;
}

bool CANSetMessageBuffer(unsigned int n, fcCC eCC, unsigned int BufferNumber,
						 fcCANBufCfg sCfg, bool ignoreTxRqstLock)
{
	PrintHeaderFunctionAndCC("fcbCANSetMessageBuffer", eCC);

	int count = 10000;
	fcError e = 0;

	while (0 != count)
	{
		e = fcbCANSetMessageBuffer(fc[n].Handle, eCC, BufferNumber, sCfg,
			ignoreTxRqstLock);
		if (NONE == fcGetErrorCode(e))
		{
			fprintf(g_pFile, "configure buffer OK\n");
			return true;
		}
		else if (MSG_BUF_LOCKED_FOR_TRANSMISSION != fcGetErrorCode(e))
		{
			PrintError(e, "fcbCANSetMessageBuffer");
			return false;
		}
		else
		{
			fprintf(g_pFile,"fcbCANSetMessageBuffer, try again\n");
			// still MSG_BUF_LOCKED_FOR_TRANSMISSION, busy wait and try again
			--count;
		}		
	}
	fprintf(g_pFile, "MSG_BUF_LOCKED_FOR_TRANSMISSION too long\n");
	return false;
}

bool CANMonitoringStart(unsigned int n, fcCC eCC, bool bResetTimeStamp,
						fcCANMonitoringMode eMonitoringMode)
{
	PrintHeaderFunctionAndCC("fcbCANMonitoringStart", eCC);

	fcError e = fcbCANMonitoringStart(fc[n].Handle, eCC, bResetTimeStamp,
		eMonitoringMode);
	if (0 == e)
	{
		fc[n].MonitoringStarted_CAN[eCC] = true;
		fprintf(g_pFile, "Ok.\n"); 
		return true;
	}
	else 
	{
		PrintError(e, "fcbCANMonitoringStart");
	}
	return false;
}

bool CANMonitoringStop(unsigned int n, fcCC eCC)
{
	//PrintHeaderFunctionAndCC("fcbCANMonitoringStop", eCC);
	fcError e = fcbCANMonitoringStop(fc[n].Handle, eCC);
	if (0 == e)
	{
		fc[n].MonitoringStarted_CAN[eCC] = false;
		//fprintf(g_pFile, "Ok.\n");
		return true;
	}
	else 
	{
		PrintError(e, "fcbCANMonitoringStop");
	}
	return false;
}


bool CANFDTransmit(unsigned int n, fcCC eCC, fcCANFDTxFrame* pFrame)
{
	PrintHeaderFunctionAndCC("fcbCANFDTransmit", eCC);

	fcError e = fcbCANFDTransmit(fc[n].Handle, eCC, pFrame);
	if (NONE == fcGetErrorCode(e))
	{
		fprintf(g_pFile,"transmit OK\n");
		return true;
	}
	else
	{
		PrintError(e, "fcbCANFDTransmit");
		return false;
	}	

}



bool CANTransmit(unsigned int n, fcCC eCC, unsigned int BufferNumber,
				 fcByte Data[8], bool bTransmitNewData, bool ignoreTxRqstLock)
{
	PrintHeaderFunctionAndCC("fcbCANTransmit", eCC);

	int count = 10000;
	fcError e = 0;

	while (0 != count)
	{
		e = fcbCANTransmit(fc[n].Handle, eCC, BufferNumber, Data,
			bTransmitNewData, ignoreTxRqstLock);
		if (NONE == fcGetErrorCode(e))
		{
			fprintf(g_pFile,"transmit OK\n");
			return true;
		}
		else if (MSG_BUF_LOCKED_FOR_TRANSMISSION != fcGetErrorCode(e))
		{
			PrintError(e, "fcbCANTransmit");
			return false;
		}
		else
		{
			fprintf(g_pFile,"CANTransmit, try again\n");
			// still MSG_BUF_LOCKED_FOR_TRANSMISSION, busy wait and try again
			--count;
		}		
	}
	fprintf(g_pFile, "MSG_BUF_LOCKED_FOR_TRANSMISSION too long\n");
	return false;
}


