#ifndef _FC_H_
#define _FC_H_

#include "stdafx.h"

#include <map>

// DEFINEs and CONSTs
const unsigned int MAXIMUMFLEXCARD = 8;
const unsigned int MAXIMUMCCS_CAN = 8;

typedef struct s_item
{
	unsigned int FlagId;
	char Text[1024];
} t_item;

static t_item gCANError[] = 
{
	{fcCANErrorNone       ,"None"},
	{fcCANErrorStuff      ,"StuffError"},
	{fcCANErrorForm       ,"FormError"},
	{fcCANErrorAcknowledge,"AcknowledgeError"},
	{fcCANErrorBit1       ,"Bit1Error"},
	{fcCANErrorBit0       ,"Bit0Error"},
	{fcCANErrorCRC        ,"CRCError"},
	{fcCANErrorParity     ,"ParityError"},
};
const unsigned int sizeCANError = sizeof(gCANError) / sizeof(gCANError[0]);

static t_item gCANState[] = 
{
	{fcCANCcStateUnknown,		"Unknown"},
	{fcCANCcStateConfig,		"Config"},
	{fcCANCcStateNormalActive,	"NormalActive"},
	{fcCANCcStateWarning,		"WarningLimit"},
	{fcCANCcStateErrorPassive,	"ErrorPassive"},
	{fcCANCcStateBusOff,		"BusOff"},
};
const unsigned int sizeCANState = sizeof(gCANState) / sizeof(gCANState[0]);


static std::map<fcBusType, std::string>  busTypeMapping =
{
    {fcBusTypeFlexRay,"FlexRay"},
    {fcBusTypeCAN, "CAN"},
    {fcBusTypeCANFD, "CAN FD"},
    {fcBusTypeEth,"Ethernet"}
};

static std::map<fcFlexCardDeviceId, std::string> deviceName =
{
    {fcNoDevice, "fcNoDevice"},
    {fcFlexCardCycloneII, "FlexCard Cyclone II"},
    {fcFlexCardCycloneIIPXI, "FlexCard PXI"},
    {fcFlexCardPMC, "FlexCard PMC"},
    {fcFlexCardCycloneIISE, "FlexCard Cyclone II SE"},
    {fcFlexCardPMCII, "FlexCard PMC-II"},
    {fcFlexXConMidgetBinderV1, "FlexXCon Midget BinderV1"},
    {fcFlexXConMidgetBinderV2, "FlexXCon Midget BinderV2"},
    {fcFlexXConMidgetLemo, "FlexXCon Midget Lemo"},
    {fcFlexXConExpert, "FlexXCon Expert"},
    {fcFlexCardUSB_M, "FlexCard USB-M"},
    {fcFlexXConCompact, "fcFlexXConCompact" },
    {frPCIeNGP, "frPCIeNGP" },
    {fcFlexDevice_L_0A01, "fcFlexDevice_L_0A01" },
    {fcFlexDevice_L_2SOC_0S01, "fcFlexDevice_L_2SOC_0S01" },
    {fcFlexDevice_L_2SOC_HDSUB_AS_0S01, "fcFlexDevice_L_2SOC_HDSUB_AS_0S01" },
    {fcFlexDevice_L_0A02, "fcFlexDevice_L_0A02" },
    {fcFlexDevice_L_2SOC_0S02, "fcFlexDevice_L_2SOC_0S02" },
    {fcFlexDevice_L_2SOC_HDSUB_AS_0S02, "fcFlexDevice_L_2SOC_HDSUB_AS_0S02" },
    {fcFlexDevice_S_0A01, "fcFlexDevice_S_0A01" },
    {fcFlexCardPXIe3, "fcFlexCardPXIe3" },
    {fcFlexCardPCIe3, "fcFlexCardPCIe3" },
};

static std::map<fcCCType, std::string> ccTypeName =
{
    {Undefined, "Undefined"},
    {FreeScale_FPGA, "FreeScale_FPGA"},
    {Bosch_E_Ray, "Bosch_E_Ray"},
    {Bosch_DCAN, "Bosch_DCAN"},
    {Bosch_M_CAN, "Bosch_M_CAN"},
    {Ethernet, "Ethernet"},
    {UART_16550, "UART_16550"},
};

typedef struct s_FC
{
	fcFlexCardDeviceId DeviceId;
	unsigned int Id;
    fcHandle Handle;
	bool MonitoringStarted_CAN[MAXIMUMCCS_CAN];
} t_FC;

// Helper functions
void PrintError(fcError e,const char* szFunction);
void PrintVersion(const fcVersionNumber& v);
void PrintHeader(const char* szText);
void PrintHeaderFunctionAndCC(const char* szText, fcCC CCIndex);
std::string ReadFile(const char* szPath);

// Common API Wrapper Functions
void EnumerateFlexCards();
void OpenFlexCard(unsigned int n);
void CloseFlexCard(unsigned int n);
bool Receive(unsigned int n, fcPacket** pPacket);
bool FreeMemory(fcMemoryType type, void* p);
bool GetCurrentHighResTimeStamp(unsigned int n, unsigned long long* pTimestamp);

// Termination API Wrapper Functions
bool SetBusTerminationCC(unsigned int n, fcCC eCC, bool bTermination);

// CAN API Wrapper Functions
bool CANFDSetCcConfiguration(unsigned int n, fcCC eCC, fcCANFDCcConfig pCAN);
bool CANSetCcConfiguration(unsigned int n, fcCC eCC, fcCANCcConfig cfg);
bool CANSetMessageBuffer(unsigned int n, fcCC eCC, unsigned int BufferNumber,
						 fcCANBufCfg sCfg, bool ignoreTxRqstLock);
bool CANMonitoringStart(unsigned int n, fcCC eCC, bool bResetTimeStamp,
						fcCANMonitoringMode eMonitoringMode);
bool CANMonitoringStop(unsigned int n, fcCC eCC);
bool CANFDTransmit(unsigned int n, fcCC eCC, fcCANFDTxFrame* pFrame);
bool CANTransmit(unsigned int n, fcCC eCC, unsigned int BufferNumber,
				 fcByte Data[8], bool bTransmitNewData, bool ignoreTxRqstLock);

#endif /* _FC_H_ */

// Global variables ...
extern t_FC fc[MAXIMUMFLEXCARD];
extern FILE * g_pFile;
