// INCLUDES
#include "fc.h"
#include <stdio.h>
#include <stdlib.h>     // rand function

#include <string>
#include <iostream>

#ifdef WIN32
	// Following headers are only used for the log file handling under Windows
	#include <Shlobj.h>
	#include <Shlwapi.h>
	#pragma comment(lib,"Shlwapi.lib")
#endif



// CONSTANTS
const unsigned int MAX_CHARS = 1024;
const fcCC CcIndex1 = fcCC1;	// Change to your preferred index
const fcCC CcIndex2 = fcCC2;	// Change to your preferred index



// PROTOTYPES
void CleanUp(unsigned int n);
void ProcessPackets(fcPacket* pPackets);
fcCANFDCcConfig GetCcConfig(fcFlexCardDeviceId deviceId);

FILE* g_pFile;



int main(void)
{
#ifdef WIN32
    // Under Windows, better log to the Documents folder.
    TCHAR szPath[MAX_PATH];
    if (SUCCEEDED(::SHGetFolderPath(NULL,
        CSIDL_PERSONAL | CSIDL_FLAG_CREATE,
        NULL,
        0,
        szPath)))
    {
        ::PathAppend(szPath, "fcDemoCAN_FD_log.txt");
        g_pFile = fopen(szPath, "w");
        printf("Output is logged to:\n\"%s\"\n...", szPath);
    }
#else
    g_pFile = fopen("fcDemoCAN_FD_log.txt", "w");
    printf("Output is logged to fcDemoCAN_FD_log.txt ...");
#endif

    fprintf(g_pFile, "FlexCard CAN FD Demo.\n");
    fprintf(g_pFile, "STAR COOPERATION GmbH\n");


    /************************************************************************/
    /* Open the FlexCard                                                    */
    /************************************************************************/

    EnumerateFlexCards();
    if (0 == fc[0].Id)
    {
        fprintf(g_pFile, "No FlexCards found.\n");
        printf("\nFinished.");
        return -1;
    }

    bool bCardFound = false;

    unsigned int uiDeviceIndex = 0;
    for (; 4 > uiDeviceIndex; uiDeviceIndex++)
    {
        if (0 != fc[0].Id &&
            (fcFlexCardCycloneII == fc[uiDeviceIndex].DeviceId ||
                fcFlexCardCycloneIISE == fc[uiDeviceIndex].DeviceId ||
                fcFlexCardPMC == fc[uiDeviceIndex].DeviceId ||
                fcFlexCardPMCII == fc[uiDeviceIndex].DeviceId ||
                fcFlexCardUSB == fc[uiDeviceIndex].DeviceId ||
                fcFlexCardPXIe3 == fc[uiDeviceIndex].DeviceId ||
                fcFlexCardPCIe3 == fc[uiDeviceIndex].DeviceId))
        {
            bCardFound = true;
            break;
        }
    }

    if (!bCardFound)
    {
        fprintf(g_pFile, "No FlexCard device found.\n");
        printf("\nFinished.");
        return -1;
    }

    // Using the last FlexCard found in the system

    if (fcFlexCardPMCII == fc[uiDeviceIndex].DeviceId ||
        fcFlexCardPXIe3 == fc[uiDeviceIndex].DeviceId ||
        fcFlexCardPCIe3 == fc[uiDeviceIndex].DeviceId)
    {
        // The device is supported
    }
    else
    {
        fprintf(g_pFile, "The FlexCard you use in this demo must be a FlexCard PMC-II, a FlexCard PXIe3 or a FlexCard PCIe3.\n");
        printf("\nFinished.");
        return -1;
    }


    OpenFlexCard(uiDeviceIndex);
    if (NULL == fc[uiDeviceIndex].Handle)
    {
        fprintf(g_pFile, "Open FlexCard failed.\n");
        printf("\nFinished.");
        return -1;
    }


    fcTinyInfoCollection tinyCollection;
    memset(&tinyCollection, 0, sizeof(tinyCollection));
    fcByte numberOfTinySlots;
    fcError eTiny = fcbGetTinyInfo(fc[0].Handle, &tinyCollection, &numberOfTinySlots);
    if (0 == eTiny)
    {
        for (int i = 0; i < numberOfTinySlots; i++)
        {
            fprintf(g_pFile, "Tiny slot index: %u, type: %u\n", i, tinyCollection.info[i].TinyType);
        }
    }
    else
    {
        PrintError(eTiny, "fcbGetTinyInfo");
    }


    /************************************************************************/
    /* Module Initialization                                                */
    /************************************************************************/

    // If you want to know how many CAN CCs on the hardware are usable, use the
    // application FlexUpdate or read the document "Instructions for Use".
    // Please ensure, that CcIndex1 and CcIndex2 are useable CC indexes.

    //if (fc[uiDeviceIndex].DeviceId == fcFlexCardPMCII)
    //{
    //	// Enable bus termination for both CCs. The termination API is
    //	// currently supported by FlexCard PMC/PCI and FlexCard PMC-II.
    //	if (! SetBusTerminationCC(uiDeviceIndex, CcIndex1, true))
    //	{
    //		CleanUp(uiDeviceIndex);
    //		return -1;
    //	}
    //	if (! SetBusTerminationCC(uiDeviceIndex, CcIndex2, true))
    //	{
    //		CleanUp(uiDeviceIndex);
    //		return -1;
    //	}
    //}


    const auto CcConfig = GetCcConfig(fc[uiDeviceIndex].DeviceId);

    #ifdef TIMESTAMP_USERIO
    if (fc[uiDeviceIndex].DeviceId == fcFlexCardPMCII)
    {
        fcTimeStampCfg cfg;
        memset(&cfg, 0, sizeof(cfg));
        cfg.mode = fcTimeStampModeUserIo;
        fcError eTimestamp = fcbConfigureFlexCardTimeStamp(fc[0].Handle, cfg);
        if (eTiny != 0)
        {
            PrintError(eTiny, "fcbConfigureFlexCardTimeStamp");
        }
    }
    #endif


	/************************************************************************/
	/* Configure CAN CCs                                                    */
	/************************************************************************/

	if (! CANFDSetCcConfiguration(uiDeviceIndex, CcIndex1, CcConfig))
	{
		CleanUp(uiDeviceIndex);
		return -1;
	}
	if (! CANFDSetCcConfiguration(uiDeviceIndex, CcIndex2, CcConfig))
	{
		CleanUp(uiDeviceIndex);
		return -1;
	}


	/************************************************************************/
	/* Pre-MonitoringStart                                                  */
	/************************************************************************/



    fcCANFDTxFrame frame1;
    memset(&frame1, 0, sizeof(fcCANFDTxFrame));
    frame1.DLC = 8;
    frame1.ID = 1;
    frame1.enableIDExtended = false;
    frame1.enableCanFdBitrateSwitch = false;
    frame1.enableCanFdFormat = false;
    frame1.enableTxAcknowledge = true;


    fcCANFDTxFrame frame2;
    memset(&frame2, 0, sizeof(fcCANFDTxFrame));
    frame2.DLC = 0;
    frame2.ID = 0x2000;
    frame2.enableIDExtended = true;
    frame2.enableCanFdBitrateSwitch = true;
    frame2.enableCanFdFormat = true;
    frame2.enableTxAcknowledge = true;


	/************************************************************************/
	/* Start CC Monitoring                                                  */
	/************************************************************************/

	// If you do not use the event mechanism, app polls every 10ms for new data
	const unsigned long long llTimeOutMicroSeconds = 3000LL * 1000LL;

	fprintf(g_pFile, "\n\n");
	fprintf(g_pFile, "********************************************************"
		"****************\n");
	fprintf(g_pFile, "* Demo duration will be about %I64u milliseconds.          "
		"             *\n", llTimeOutMicroSeconds / 1000LL);
	fprintf(g_pFile, "* Ensure that used CANs of the FlexCard are connected wi"
		"th each other. *\n");
	fprintf(g_pFile, "********************************************************"
		"****************\n");
	fprintf(g_pFile, "\n\n");

	fcCANMonitoringMode CcMode = fcCANMonitoringNormal;
	bool bResetTimeStamp = true;
	CANMonitoringStart(uiDeviceIndex, CcIndex1, bResetTimeStamp, CcMode);
	if (! fc[uiDeviceIndex].MonitoringStarted_CAN[CcIndex1])
	{
		CleanUp(uiDeviceIndex);
		return -1;
	}
	bResetTimeStamp = false;
	CANMonitoringStart(uiDeviceIndex, CcIndex2, bResetTimeStamp, CcMode);
	if (! fc[uiDeviceIndex].MonitoringStarted_CAN[CcIndex2])
	{ 
		CleanUp(uiDeviceIndex);
		return -1;
	}

	 Sleep(100);

	/************************************************************************/
	/* Post-MonitoringStart                                                 */
	/************************************************************************/


	fcByte loopCount = 0;

	/************************************************************************/
	/* Main loop                                                            */
	/************************************************************************/

	fprintf(g_pFile, "Start main loop\n\n");

	while (true)
	{
		++loopCount;

		
        for(int i = 0; i < 64; i++)
        {
            frame1.data[i] = (fcByte)rand();
        }
        CANFDTransmit(uiDeviceIndex, CcIndex1, &frame1);

        if(frame2.DLC < 0xF)
        {
            frame2.DLC++;
        }
        else
        {
            frame2.DLC = 1;
        }
        for(int i = 0; i < 64; i++)
        {
            frame2.data[i] = (fcByte)rand();
        }
        CANFDTransmit(uiDeviceIndex, CcIndex2, &frame2);


		Sleep(20);
		

		fcPacket* pPacket = NULL;

		// Receive the data
		if (! Receive(uiDeviceIndex, &pPacket))
			break;

		if (NULL != pPacket)
		{
			// The demo just prints the packets
			ProcessPackets(pPacket);

			// Release packet memory
			if (! FreeMemory(fcMemoryTypePacket, pPacket))
				break;
		}

		unsigned long long llTimeInMS = 0LL;
		if (! GetCurrentHighResTimeStamp(uiDeviceIndex , &llTimeInMS))
			break;

		// Loop x seconds
		if (llTimeInMS > llTimeOutMicroSeconds)
			break;
	}

	fprintf(g_pFile, "\n\nEnd main loop\n\n");
	Sleep(100);

	/************************************************************************/
	/* Pre-MonitoringStop                                                   */
	/************************************************************************/

	fcPacket* pPacketEnd = NULL;
	if (! Receive(uiDeviceIndex, &pPacketEnd))
	{
		CleanUp(uiDeviceIndex);
		return -1;
	}


	if (NULL != pPacketEnd)
	{
		// The demo just prints the packets
		ProcessPackets(pPacketEnd);

		// Release packet memory
		if (! FreeMemory(fcMemoryTypePacket, pPacketEnd))
		{
			CleanUp(uiDeviceIndex);
			return -1;
		}
	}

	CleanUp(uiDeviceIndex);
	return 0;
};







int fcdIoConvertCanFdDataSizeToNumber(fcByte dlc)
{
    int numberOfByte;

    if(dlc <= 8)
    {
        return (int)dlc;
    }

    switch (dlc)
    {
        case 9:
        {
            numberOfByte = 12;
            break;
        }
        case 10:
        {
            numberOfByte = 16;
            break;
        }
        case 11:
        {
            numberOfByte = 20;
            break;
        }
        case 12:
        {
            numberOfByte = 24;
            break;
        }
        case 13:
        {
            numberOfByte = 32;
            break;
        }
        case 14:
        {
            numberOfByte = 48;
            break;
        }
        case 15:
        {
            numberOfByte = 64;
            break;
        }
        default:
        {
            numberOfByte = 0;
            break;
        }

    }

    return numberOfByte;
}




void Print_fcCANErrorPacket(fcCANErrorPacket* pFrame)
{
	fprintf(g_pFile, "CANError    \t %f \t [%08X] \t fcCC%d \t "
		,(float)pFrame->TimeStamp * 0.000001
		,pFrame->TimeStamp
		,pFrame->CC+1) ;



	for (unsigned int i=0; i < sizeCANError; i++)
	{
		if (gCANError[i].FlagId == (unsigned int)pFrame->Type)
		{
			fprintf(g_pFile, "%s ",gCANError[i].Text);
			break;
		}
	}

	for (unsigned int i=0; i < sizeCANState; i++)
	{
		if (gCANError[i].FlagId == (unsigned int)pFrame->State)
			fprintf(g_pFile, "%s ",gCANState[i].Text);
	}

	fprintf(g_pFile, "\n");


}

void Print_fcCANFDPacket(fcCANFDPacket* pFrame)
{
	fprintf(g_pFile, " CANFDPacket   \t");


	if(pFrame->Direction)
		fprintf(g_pFile, "Tx");
	else
		fprintf(g_pFile, "Rx");

	if(pFrame->RemoteFrame)
		fprintf(g_pFile, " Remote");


    unsigned long long timestamp = (unsigned long long)(pFrame->TimeStampHigh) << 32;
    timestamp += (unsigned long long)(pFrame->TimeStampLow);
	fprintf(g_pFile, "\t %f (sec) \t fcCC%d \t"
		,(float)timestamp * 0.000001
		,pFrame->CC+1);


	if (pFrame->ExtendedId)
		fprintf(g_pFile, "Ext");
	else
		fprintf(g_pFile, "Std");

    int payloadLengthBytes = fcdIoConvertCanFdDataSizeToNumber(pFrame->DLC);

	fprintf(g_pFile, "Id: 0x%08X, Payloadlength (bytes): %d, CanFdFormat: %d, BitrateSwitch: %d, BufferNumber: %03d, "
		,pFrame->ID
        ,payloadLengthBytes
        ,pFrame->FdFormat
        ,pFrame->BitRateSwitch
		,pFrame->BufferNumber
		);

    if(pFrame->MessageLost)
		fprintf(g_pFile, "MsgLost,  ");

    if(pFrame->ESI)
		fprintf(g_pFile, "ESI 1, ");
    else
		fprintf(g_pFile, "ESI 0, ");

	for (int j=0; j < payloadLengthBytes; j++)
	{
		fprintf(g_pFile, " %02x", (int)pFrame->pData[j]);
	}

	fprintf(g_pFile, "\n");

}



void Print_fcCANFDErrorPacket(fcCANFDErrorPacket* pFrame)
{
    unsigned long long timestamp = (unsigned long long)(pFrame->TimeStampHigh) << 32;
    timestamp += (unsigned long long)(pFrame->TimeStampLow);
	fprintf(g_pFile, "CANFDError    \t %f (sec) \t fcCC%d \t "
		,(float)timestamp * 0.000001
		,pFrame->CC+1) ;



	for (unsigned int i=0; i < sizeCANError; i++)
	{
		if (gCANError[i].FlagId == (unsigned int)pFrame->Type)
		{
			fprintf(g_pFile, "%s ",gCANError[i].Text);
			break;
		}
	}

	for (unsigned int i=0; i < sizeCANState; i++)
	{
		if (gCANError[i].FlagId == (unsigned int)pFrame->State)
			fprintf(g_pFile, "%s ",gCANState[i].Text);
	}

    fprintf(g_pFile, "RxErrorCounter: %u TxErrorCounter: %u\n", pFrame->ReceiveErrorCounter, pFrame->TransmitErrorCounter);

}

void Print_fcCANPacket(fcCANPacket* pFrame)
{
	fprintf(g_pFile, " CANPacket   \t");


	if(pFrame->Direction)
		fprintf(g_pFile, "Tx");
	else
		fprintf(g_pFile, "Rx");

	if(pFrame->RemoteFrame)
		fprintf(g_pFile, " Remote");



	fprintf(g_pFile, "\t %f \t [0x%08X] \t fcCC%d \t"
		,(float)pFrame->TimeStamp * 0.000001
		,pFrame->TimeStamp
		,pFrame->CC+1);


	if (pFrame->ExtendedId)
		fprintf(g_pFile, "Ext");
	else
		fprintf(g_pFile, "Std");
	fprintf(g_pFile, "Id: 0x%08X BufferNumber: %03d DLC: %d "
		,pFrame->ID
		,pFrame->BufferNumber
		,pFrame->DLC);

	unsigned int len = pFrame->DLC;
	for (unsigned int j=0; j < len; j++)
	{
		fprintf(g_pFile, " %02x", (int)pFrame->Data[j]);
	}

	if(pFrame->MessageLost)
		fprintf(g_pFile, "MsgLost");

	fprintf(g_pFile, "\n");

}

void ProcessPackets(fcPacket* pPackets)
{
	fcPacket* p = pPackets;
	while (NULL != p)
	{
		switch (p->Type)
		{
        case fcPacketTypeCANFD:
			Print_fcCANFDPacket(p->CANFDPacket);
			break;
		case fcPacketTypeCANFDError:
			Print_fcCANFDErrorPacket(p->CANFDErrorPacket);
			break;
		case fcPacketTypeCAN:
			Print_fcCANPacket(p->CANPacket);
			break;
		case fcPacketTypeCANError:
			Print_fcCANErrorPacket(p->CANErrorPacket);
			break;

		default:
			fprintf(g_pFile, "\n Unknown Packet!!!");
		};

		// get the next packet
		p = p->pNextPacket;
	}
}

void CleanUp(unsigned int n)
{
	if (MAXIMUMFLEXCARD <= n)
	{
		fprintf(g_pFile, "Finished.");
		fflush(g_pFile);
		fclose(g_pFile);
		return;
	}

	if (fc[n].MonitoringStarted_CAN[CcIndex1])
	{
		CANMonitoringStop(n, CcIndex1);
	}
	if (fc[n].MonitoringStarted_CAN[CcIndex2])
	{
		CANMonitoringStop(n, CcIndex2);
	}

	//if (fc[n].DeviceId == fcFlexCardPMC ||
	//	fc[n].DeviceId == fcFlexCardPMCII)
	//{
	//	// Disable bus termination for both CCs
	//	SetBusTerminationCC(n, CcIndex1, false);
	//	SetBusTerminationCC(n, CcIndex2, false);
	//}

	CloseFlexCard(n);

	fprintf(g_pFile, "Finished.");
	fflush(g_pFile);
	fclose(g_pFile);

	printf("\nFinished.");

	return;
}


fcCANFDCcConfig GetCcConfig(fcFlexCardDeviceId deviceId)
{
    fcCANFDCcConfig config = {};

    if (deviceId == fcFlexCardPMCII)
    {
        // 500 KBit/s nominal
        config.nominalBitTime.BaudRatePrescaler = 1;
        config.nominalBitTime.SynchronizationJumpWidth = 1;
        config.nominalBitTime.TimeSegment1 = 29;
        config.nominalBitTime.TimeSegment2 = 10;


        // 4 MBit/s data
        config.dataBitTime.BaudRatePrescaler = 1;
        config.dataBitTime.SynchronizationJumpWidth = 1;
        config.dataBitTime.TimeSegment1 = 2;
        config.dataBitTime.TimeSegment2 = 2;

        config.frameFormat = fcCANFDFormatIso11898_1;
        config.enableAutomaticRetransmission = true; // If there is a lot of
        // CAN traffic and the buffer could not be sent, automatically try again
        config.enableCANFDOperation = true;
        config.enableCANFDBitRateSwitch = true;


    }
    else
    {
        // FlexCard PXIe3/PCIe3

        // 500 KBit/s nominal
        config.nominalBitTime.BaudRatePrescaler = 5;
        config.nominalBitTime.SynchronizationJumpWidth = 4;
        config.nominalBitTime.TimeSegment1 = 11;
        config.nominalBitTime.TimeSegment2 = 4;

        // 2 MBit/s data
        config.dataBitTime.BaudRatePrescaler = 1;
        config.dataBitTime.SynchronizationJumpWidth = 4;
        config.dataBitTime.TimeSegment1 = 14;
        config.dataBitTime.TimeSegment2 = 5;

        config.frameFormat = fcCANFDFormatIso11898_1;
        config.enableAutomaticRetransmission = true; // If there is a lot of
        // CAN traffic and the buffer could not be sent, automatically try again
        config.enableCANFDOperation = true;
        config.enableCANFDBitRateSwitch = true;

    }

    return config;
}
